@@ -3,6 +3,7 @@ require 'net/ssh/ruby_compat'
 require 'net/ssh/connection/channel'
 require 'net/ssh/connection/constants'
 require 'net/ssh/service/forward'
+require 'net/ssh/connection/keepalive'
 
 module Net; module SSH; module Connection
 
@@ -79,7 +80,7 @@ module Net; module SSH; module Connection
       @max_pkt_size = (options.has_key?(:max_pkt_size) ? options[:max_pkt_size] : 0x8000)
       @max_win_size = (options.has_key?(:max_win_size) ? options[:max_win_size] : 0x20000)
 
-      @last_keepalive_sent_at = nil
+      @keepalive = Keepalive.new(self)
     end
 
     # Retrieves a custom property from this instance. This can be used to
@@ -244,7 +245,7 @@ module Net; module SSH; module Connection
         writer.send_pending
       end
 
-      send_keepalive_as_needed(readers, writers)
+      @keepalive.send_as_needed(readers, writers)
       transport.rekey_as_needed
 
       return true
@@ -350,19 +351,21 @@ module Net; module SSH; module Connection
     end
 
     # Same as #exec, except this will block until the command finishes. Also,
-    # if a block is not given, this will return all output (stdout and stderr)
+    # if no block is given, this will return all output (stdout and stderr)
     # as a single string.
     #
     #   matches = ssh.exec!("grep something /some/files")
     def exec!(command, &block)
-      block ||= Proc.new do |ch, type, data|
+      block_or_concat = block || Proc.new do |ch, type, data|
         ch[:result] ||= ""
         ch[:result] << data
       end
 
-      channel = exec(command, &block)
+      channel = exec(command, &block_or_concat)
       channel.wait
 
+      channel[:result] ||= "" unless block
+
       return channel[:result]
     end
 
@@ -598,27 +601,8 @@ module Net; module SSH; module Connection
 
       def io_select_wait(wait)
         return wait if wait
-        return wait unless options[:keepalive]
-        keepalive_interval
-      end
-
-      def keepalive_interval
-        options[:keepalive_interval] || DEFAULT_IO_SELECT_TIMEOUT
-      end
-
-      def should_send_keepalive?
-        return false unless options[:keepalive]
-        return true unless @last_keepalive_sent_at
-        Time.now - @last_keepalive_sent_at >= keepalive_interval
-      end
-
-      def send_keepalive_as_needed(readers, writers)
-        return unless readers.nil? && writers.nil?
-        return unless should_send_keepalive?
-        info { "sending keepalive" }
-        msg = Net::SSH::Buffer.from(:byte, Packet::IGNORE, :string, "keepalive")
-        send_message(msg)
-        @last_keepalive_sent_at = Time.now
+        return wait unless @keepalive.enabled?
+        @keepalive.interval
       end
 
       MAP = Constants.constants.inject({}) do |memo, name|
